@@ -12,30 +12,34 @@ __all__ = [
     "Report",
     "Program",
 ]
+import json
+import logging
 import urllib.parse as urlparse
 from io import BytesIO
 from typing import List
 
 import requests
-import json
-import pprint
-import logging
-from .models import Attr, YwhApiObject
+
 from .exceptions import (
     APIError,
-    InvalidResponse,
-    ObjectNotFound,
+    AccessDenied,
     BadCredentials,
-    TOTPLoginEnabled,
-    JWTNotFound,
+    InvalidResponse,
     JWTInvalid,
+    JWTNotFound,
     OAuth2CodeError,
-    AccessDenied
+    ObjectNotFound,
+    TOTPLoginEnabled,
 )
+from .models import (
+    Attr,
+    YwhApiObject,
+)
+
 logger = logging.getLogger(__name__)
 
 class YesWeHack(YwhApiObject):
-    #  attributes item
+    # attributes item
     _attrs = [
         Attr(name="token", type_=str),
         Attr(name="api_url", type_=str, default="https://api.yeswehack.com"),
@@ -47,7 +51,8 @@ class YesWeHack(YwhApiObject):
         Attr(name='oauth_args', type_=dict, default={}),
         Attr(name='session', type_=object),
         Attr(name='apps_headers', type_=object, default={}),
-        Attr(name='managed_pgms', type_=list, default=[])
+        Attr(name='managed_pgms', type_=list, default=[]),
+        Attr(name='pat', type_=str, default=None),
     ]
 
     def __init__(self, headers={}, totp_code=None, **kwargs):
@@ -58,6 +63,8 @@ class YesWeHack(YwhApiObject):
         self.session = requests.session()
         self.session.verify = self.verify
         self.session.headers = headers
+        if self.pat:
+            self.session.headers['X-AUTH-TOKEN'] = self.pat
 
         if not self.lazy and self.username != None and self.password != None:
             self.login(totp_code=totp_code)
@@ -72,7 +79,7 @@ class YesWeHack(YwhApiObject):
         try:
             json_resp = resp.json()
         except Exception as e:
-            raise InvalidResponse
+            raise InvalidResponse(message=resp.text) from e
 
         if resp.status_code != 200:
             if resp.status_code == 401:
@@ -151,6 +158,8 @@ class YesWeHack(YwhApiObject):
         return False
 
     def login(self, totp_code=None):
+        if self.pat:
+            return  # no login with PAT
         if self.oauth_mode:
             return self._oauth_login()
         else:
@@ -210,7 +219,7 @@ class YesWeHack(YwhApiObject):
             resp.raise_for_status()
             json_resp = resp.json()
         except Exception as e:
-            raise InvalidResponse from e
+            raise InvalidResponse(message=resp.text) from e
         return Log(ywh_api=self, **json_resp)
 
     def managed_programs(self, lazy=False):
@@ -471,7 +480,7 @@ class Report(YwhApiObject):
             id=self.id, export_format=export_format
         )
         resp = self.ywh_api.raw_call(
-            "GET", self.ywh_api.api_url + path, headers=self.ywh_api.headers
+            "GET", self.ywh_api.api_url + path
         )
         if resp.status_code != 200:
             print(
@@ -489,8 +498,7 @@ class Report(YwhApiObject):
 
         if message is not None:
             datas['message'] = message
-        headers = {**self.ywh_api.session.headers, **self.ywh_api.apps_headers}
-        resp = self.ywh_api.raw_call('PUT', f'{self.ywh_api.api_url}/reports/{self.id}/tracking-status', data=json.dumps(datas), headers=headers)
+        resp = self.ywh_api.raw_call('PUT', f'{self.ywh_api.api_url}/reports/{self.id}/tracking-status', data=json.dumps(datas), headers=self.ywh_api.apps_headers)
         return resp
 
     def post_tracker_update(self, tracker_name, tracker_url, tracker_id=None, token=None, message=None):
@@ -502,8 +510,7 @@ class Report(YwhApiObject):
         }
         if message is not None:
             data['message'] = message
-        headers = {**self.ywh_api.session.headers, **self.ywh_api.apps_headers}
-        resp = self.ywh_api.raw_call('POST', f'{self.ywh_api.api_url}/reports/{self.id}/tracker-update', data=json.dumps(data), headers=headers)
+        resp = self.ywh_api.raw_call('POST', f'{self.ywh_api.api_url}/reports/{self.id}/tracker-update', data=json.dumps(data), headers=self.ywh_api.apps_headers)
         return resp
 
     def post_tracker_message(self, tracker_name, tracker_url, tracker_id=None, message=None, attachments=None) -> Log:
@@ -515,12 +522,11 @@ class Report(YwhApiObject):
         }
         if message is not None:
             data['message'] = message
-        headers = {**self.ywh_api.session.headers, **self.ywh_api.apps_headers}
         http_response = self.ywh_api.raw_call(
             method='POST',
             url=f'{self.ywh_api.api_url}/reports/{self.id}/tracker-message',
             data=json.dumps(data),
-            headers=headers,
+            headers=self.ywh_api.apps_headers,
         )
         if not http_response.ok:
             raise APIError(f'Unable to send tracker message. Response: {http_response.text}')
@@ -534,14 +540,13 @@ class Report(YwhApiObject):
         return Log(ywh_api=self.ywh_api, **content)
 
     def post_attachment(self, filename: str, file_content: bytes, file_type: str = None, lazy: bool = False) -> Attachment:
-        headers = {**self.ywh_api.session.headers, **self.ywh_api.apps_headers}
         http_response = self.ywh_api.raw_call(
             method='POST',
             url=f'{self.ywh_api.api_url}/reports/{self.id}/attachments',
             files={
                 'file': (filename, BytesIO(file_content), file_type)
             },
-            headers=headers,
+            headers=self.ywh_api.apps_headers,
         )
         if not http_response.ok:
             raise APIError(f'Unable to create attachment. Response: {http_response.text}')
@@ -566,12 +571,11 @@ class Report(YwhApiObject):
         }
         if message is not None:
             data['message'] = message
-        headers = {**self.ywh_api.session.headers, **self.ywh_api.apps_headers}
         http_response = self.ywh_api.raw_call(
             method='PUT',
             url=f'{self.ywh_api.api_url}/reports/{self.id}/status',
             data=json.dumps(data),
-            headers=headers,
+            headers=self.ywh_api.apps_headers,
         )
         if not http_response.ok:
             raise APIError(f'Unable to send report status. Response: {http_response.text}')
